
    subroutine CanonicalTripletResonances (gz,gy,gx,dd,ee,bmin,bmax,freq,ilevel,jlevel,zl,yl,xl,bres,aperp,aparl)
    
!	WRHagen	20200823

	
  !DEC$ ATTRIBUTES DLLEXPORT::CanonicalTripletResonances
  !DEC$ ATTRIBUTES STDCALL::CanonicalTripletResonances
  IMPLICIT NONE

  ! I/O varibles
    REAL	:: gz,gy,gx,dd,ee,bmin,bmax,freq,zl,yl,xl,bres(2)
    INTEGER :: ilevel,jlevel

  ! Variables

    INTEGER	:: i,j,k,l,j1,j2,ksim,ierr,m,kv,lv
    
	REAL(8)	:: machep
	REAL(8)	:: emat(3,3), zmat(3,3), areal(3,3), aimgy(3,3)
	REAL(8)	:: diago(3), offdi(3), dummy(3), tau(3,3)
	REAL(8)	:: vecre(3,3), vecim(3,3)
    COMPLEX :: vcmplx(3,3)

    REAL :: energy,eprev,brange,bstep,hanu,d,e,b
    
	REAL	:: bb(32),en(32,3),en2(32,3),er(1024,3)
	REAL	:: edum(32),edum2(32)
    REAL	:: perp, parl, aperp(2), aparl(2)

	
    ! Body of Triplet

	ksim = 1024
    brange	= bmax - bmin
	bstep	= brange / (REAL(ksim)-1.)

!	What is the smallest number on this computer (for matrix diagonalization):
	CALL machine_precision (machep)

!	dd, ee are (my) universal parameters for comparison with S = 2, 3, 4, etc.
	d		= dd / 3.
	e		= ee

    hanu	= (0.33361E-4) * freq
    
!	Construct the field-independent matrices emat and zmat:
		CALL PREM2H (gz,gy,gx,d,e,zl,yl,xl,emat,zmat)


!	Exactly calculate a 32*3 grid of energy levels over the field range:
		k = 32
        DO      i = 1, k
                bb(i) = bmin + (i-1) * ( brange / (k-1.) )
                CALL DIAM2H (bb(i),emat,zmat,areal,aimgy)
                DO      j1 = 1, 3
                        DO      j2 = 1, 3
                                vecre(j1,j2) = 0.
                                vecim(j1,j2) = 0.
                        END DO
                        vecre(j1,j1) = 1.
                END DO
                CALL HTRIDI (3,3,areal,aimgy,diago,offdi,dummy,tau)
                CALL TQL2 (machep,3,3,diago,offdi,vecre,ierr)
                DO      j = 1, 3
                        en(i,j) = diago(j)
                END DO
        END DO

!	Spline-interpolate ksim * 3 energy levels over the field range:
        DO      j = 1, 3
                DO      i = 1, k
                        edum(i) = en(i,j)
                END DO
                CALL SPLIN0 (bb,edum,k,edum2)
                DO      i = 1, k
                        en2(i,j) = edum2(i)
                END DO
        END DO
        DO      j = 1, 3
                DO      i = 1, k
                        edum (i) = en (i,j)
                        edum2(i) = en2(i,j)
                END DO
                DO      m = 1, ksim
                        b = bmin + (m-1.) * bstep
                        CALL SPLINT (bb,edum,edum2,k,b,energy)
                        er (m,j) = energy
                END DO
        END DO

!	Compute resonance conditions as fields bvalM for which Ej-Ei-hanu=0 :

    eprev = er(1,jlevel) - er(1,ilevel) - hanu + 2. * machep
    l = 1
    DO      m = 2, ksim
            energy = er(m,jlevel) - er(m,ilevel) - hanu
            IF ( (ABS(energy).LE.1.E-6).OR.((energy / eprev).LT.0.) ) THEN
                    bres(l) = bmin+(m-1.5)*bstep
                    l = 2
            END IF
            eprev = energy + 2. * machep
    END DO
 
!	Calculate parlM and perpM transition probs at resonance fields bres(1):
    
    DO	k=1,2
		aperp(k) = 0.
        aparl(k) = 0.
    END DO
    DO	k=1,2
		IF (bres(k) .GT. 0) THEN    
			CALL DIAM2H (bres(1),emat,zmat,areal,aimgy)
			DO  j1 = 1, 3
				DO  j2 = 1, 3
					vecre(j1,j2) = 0.
					vecim(j1,j2) = 0.
				END DO
				vecre(j1,j1) = 1.
			END DO
			CALL HTRIDI (3,3,areal,aimgy,diago,offdi,dummy,tau)
			CALL TQL2   (machep,3,3,diago,offdi,vecre,ierr)
			CALL HTRIBK (3,3,areal,aimgy,tau,3,vecre,vecim)
			DO  kv = 1, 3
				DO  lv = 1, 3
				vcmplx(kv,lv)=(vecre(kv,lv)+(0.,1.)*vecim(kv,lv))
				END DO
			END DO
			CALL INTNS2H (ilevel,jlevel,xl,yl,zl,perp,parl,gx,gy,gz,vcmplx)
			aperp(k) = perp
			aparl(k) = parl
        END IF
    END DO

	RETURN
	
    end subroutine CanonicalTripletResonances
!
!
!
! = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
	SUBROUTINE intns2H (i,j,xl,yl,zl,perp,parl,gx,gy,gz,cm)
!
! @ W.R.Hagen 970301; 970702
!
	COMPLEX cm(3,3),smin,spls,sz,sy,sx

	s2      = 1.4142136

	smin =  s2*CONJG(cm(3,i))*cm(1,j) + s2*CONJG(cm(2,i))*cm(3,j)

	spls =  s2*CONJG(cm(1,i))*cm(3,j) + s2*CONJG(cm(3,i))*cm(2,j)

	sz =       CONJG(cm(1,i))*cm(1,j) -    CONJG(cm(2,i))*cm(2,j)

	sx =    (spls + smin) / 2.
	sy =    (spls - smin) / 2. / (0., 1.)

	sx =    sx * gx
	sy =    sy * gy
	sz =    sz * gz

	CALL intens (sx,sy,sz,xl,yl,zl,perp,parl)

	RETURN
	END
!
!
!
! = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
	SUBROUTINE intens (sx,sy,sz,xl,yl,zl,perp,parl)
!
! @ W.R.Hagen 930926: intens contains the part common to all
! intensity calculating subroutines intnsN.
!
	COMPLEX sx,sy,sz,ccsx,ccsy,ccsz,cparl,cperp

	ccsx =  CONJG (sx)
	ccsy =  CONJG (sy)
	ccsz =  CONJG (sz)

	cparl =         xl * xl  *  ccsx * sx
	cparl = cparl + yl * yl  *  ccsy * sy
	cparl = cparl + zl * zl  *  ccsz * sz
	cparl = cparl + xl * yl  * (ccsx * sy + ccsy * sx)
	cparl = cparl + xl * zl  * (ccsx * sz + ccsz * sx)
	cparl = cparl + yl * zl  * (ccsy * sz + ccsz * sy)

	cperp =         (1. - xl * xl) *  ccsx * sx
	cperp = cperp + (1. - yl * yl) *  ccsy * sy
	cperp = cperp + (1. - zl * zl) *  ccsz * sz
	cperp = cperp - xl * yl  * (ccsx * sy + ccsy * sx)
	cperp = cperp - xl * zl  * (ccsx * sz + ccsz * sx)
	cperp = cperp - yl * zl  * (ccsy * sz + ccsz * sy)

	perp =  REAL (cperp)
	perp =  perp / 2.
	perp =  ABS  ( perp)

	parl =  REAL (cparl)
	parl =  ABS  ( parl)

	RETURN
	END
!
!
!
! = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
      SUBROUTINE HTRIBK(NM,N,AR,AI,TAU,M,ZR,ZI)
!
      INTEGER I,J,K,L,M,N,NM
      REAL*8 AR(NM,N),AI(NM,N),TAU(2,N),ZR(NM,M),ZI(NM,M)
      REAL*8 H,S,SI
!
      IF (M .EQ. 0) GO TO 200
!
      DO 50 K = 1, N
!
	 DO 50 J = 1, M
	    ZI(K,J) = -ZR(K,J) * TAU(2,K)
	    ZR(K,J) = ZR(K,J) * TAU(1,K)
   50 CONTINUE
!
      IF (N .EQ. 1) GO TO 200
!
      DO 140 I = 2, N
	 L = I - 1
	 H = AI(I,I)
	 IF (H .EQ. 0.0D0) GO TO 140
!
	 DO 130 J = 1, M
	    S = 0.0D0
	    SI = 0.0D0
!
	    DO 110 K = 1, L
	       S = S + AR(I,K) * ZR(K,J) - AI(I,K) * ZI(K,J)
	       SI = SI + AR(I,K) * ZI(K,J) + AI(I,K) * ZR(K,J)
  110       CONTINUE
!
	    S = (S / H) / H
	    SI = (SI / H) / H
!
	    DO 120 K = 1, L
	       ZR(K,J) = ZR(K,J) - S * AR(I,K) - SI * AI(I,K)
	       ZI(K,J) = ZI(K,J) - SI * AR(I,K) + S * AI(I,K)
  120       CONTINUE
!
  130    CONTINUE
!
  140 CONTINUE
!
  200 RETURN
      END


! = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
	SUBROUTINE machine_precision (x)

! @ W.R.Hagen  891018
! This subroutine computes a floating-point precision criterion
! to be used in SUBROUTINE TQL2 (matrix diagonalization).

	REAL*8 x

	x = 1.
	DO      i = 1, 100
		x = x / 2.
		a = 1 + x
		IF (a .EQ. 1.) RETURN
	END DO
	RETURN
	END
!
!
!
! = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
	SUBROUTINE PREM2H (gz,gy,gx,d,e,zl,yl,xl,emat,zmat)
!
! @ W.R.Hagen 970301
	
	REAL*8  emat(3,3), zmat(3,3)

	bet     = 0.466882E-4
	sq2     = 1.4142136 / 2.

	DO 10   i = 1, 3
	DO 10   j = 1, 3
		zmat(i,j) = 0.
 10             emat(i,j) = 0.

! Unit-field Zeemman factors:
	zg      = gz * bet * zl
	yg      = gy * bet * yl
	xg      = gx * bet * xl

! Real zero-field elements:
	zmat (1,1) = d
	zmat (2,2) = d
	zmat (3,3) = -2.*d
	zmat (2,1) = e

! Real unit-field Zeeman elements:
	emat (1,1) =  zg
	emat (3,1) =  xg * sq2
	emat (2,2) = -zg
	emat (3,2) =  xg * sq2

! Complex unit-field Zeeman elemets:
	emat (1,3) =  yg * sq2
	emat (2,3) = -yg * sq2

	RETURN
	END
!
!
!
! = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
	SUBROUTINE DIAM2H (bval,emat,zmat,areal,aimgy)
!
! @ W.R.Hagen 970301
!
	REAL*8  emat(3,3),zmat(3,3),areal(3,3),aimgy(3,3)

	DO 10   i = 1, 3
	DO 10   j = 1, i
		areal (i,j) = emat (i,j)
 10             aimgy (i,j) = 0.

	areal (1,1) = areal (1,1) * bval
	areal (3,1) = areal (3,1) * bval
	areal (2,2) = areal (2,2) * bval
	areal (3,2) = areal (3,2) * bval

	DO 20   i = 1, 3
	DO 20   j = 1, i
		areal (i,j) = areal (i,j) + zmat (i,j)
 20     CONTINUE

	aimgy (3,1) = emat (1,3) * bval
	aimgy (3,2) = emat (2,3) * bval

	RETURN
	END
!
!
!
! = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =

! = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
      SUBROUTINE HTRIDI(NM,N,AR,AI,D,E,E2,TAU)
!
      INTEGER I,J,K,L,N,II,NM,JP1
      REAL*8 AR(NM,N),AI(NM,N),D(N),E(N),E2(N),TAU(2,N)
      REAL*8 F,G,H,FI,GI,HH,SI,SCALE
      REAL*8 DSQRT,DABS
!
      TAU(1,N) = 1.0D0
      TAU(2,N) = 0.0D0
!
      DO 100 I = 1, N
  100 D(I) = AR(I,I)
!
      DO 300 II = 1, N
	 I = N + 1 - II
	 L = I - 1
	 H = 0.0D0
	 SCALE = 0.0D0
	 IF (L .LT. 1) GO TO 130
!
	 DO 120 K = 1, L
  120    SCALE = SCALE + DABS(AR(I,K)) + DABS(AI(I,K))
!
	 IF (SCALE .NE. 0.0D0) GO TO 140
	 TAU(1,L) = 1.0D0
	 TAU(2,L) = 0.0D0
  130    E(I) = 0.0D0
	 E2(I) = 0.0D0
	 GO TO 290
!
  140    DO 150 K = 1, L
	    AR(I,K) = AR(I,K) / SCALE
	    AI(I,K) = AI(I,K) / SCALE
	    H = H + AR(I,K) * AR(I,K) + AI(I,K) * AI(I,K)
  150    CONTINUE
!
	 E2(I) = SCALE * SCALE * H
	 G = DSQRT(H)
	 E(I) = SCALE * G
	 F = DSQRT(AR(I,L)**2+AI(I,L)**2)
!
	 IF (F .EQ. 0.0D0) GO TO 160
	 TAU(1,L) = (AI(I,L) * TAU(2,I) - AR(I,L) * TAU(1,I)) / F
	 SI = (AR(I,L) * TAU(2,I) + AI(I,L) * TAU(1,I)) / F
	 H = H + F * G
	 G = 1.0D0 + G / F
	 AR(I,L) = G * AR(I,L)
	 AI(I,L) = G * AI(I,L)
	 IF (L .EQ. 1) GO TO 270
	 GO TO 170
  160    TAU(1,L) = -TAU(1,I)
	 SI = TAU(2,I)
	 AR(I,L) = G
  170    F = 0.0D0
!
	 DO 240 J = 1, L
	    G = 0.0D0
	    GI = 0.0D0
!
	    DO 180 K = 1, J
	       G = G + AR(J,K) * AR(I,K) + AI(J,K) * AI(I,K)
	       GI = GI - AR(J,K) * AI(I,K) + AI(J,K) * AR(I,K)
  180       CONTINUE
!
	    JP1 = J + 1
	    IF (L .LT. JP1) GO TO 220
!
	    DO 200 K = JP1, L
	       G = G + AR(K,J) * AR(I,K) - AI(K,J) * AI(I,K)
	       GI = GI - AR(K,J) * AI(I,K) - AI(K,J) * AR(I,K)
  200       CONTINUE
!
  220       E(J) = G / H
	    TAU(2,J) = GI / H
	    F = F + E(J) * AR(I,J) - TAU(2,J) * AI(I,J)
  240    CONTINUE
!
	 HH = F / (H + H)
!
	 DO 260 J = 1, L
	    F = AR(I,J)
	    G = E(J) - HH * F
	    E(J) = G
	    FI = -AI(I,J)
	    GI = TAU(2,J) - HH * FI
	    TAU(2,J) = -GI
!
	    DO 260 K = 1, J
	       AR(J,K) = AR(J,K) - F * E(K) - G * AR(I,K) + FI * TAU(2,K) + GI * AI(I,K)
	       AI(J,K) = AI(J,K) - F * TAU(2,K) - G * AI(I,K) - FI * E(K) - GI * AR(I,K)
  260    CONTINUE
!
  270    DO 280 K = 1, L
	    AR(I,K) = SCALE * AR(I,K)
	    AI(I,K) = SCALE * AI(I,K)
  280    CONTINUE
!
	 TAU(2,L) = -SI
  290    HH = D(I)
	 D(I) = AR(I,I)
	 AR(I,I) = HH
	 AI(I,I) = SCALE * DSQRT(H)
  300 CONTINUE
!
      RETURN
      END
!
!
!
! = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
      SUBROUTINE TQL2(machep,NM,N,D,E,Z,IERR)
!
      INTEGER I,J,K,L,M,N,II,L1,NM,MML,IERR
      REAL*8 D(N),E(N),Z(NM,N)
      REAL*8 B,C,F,G,H,P,R,S,MACHEP
      REAL*8 DSQRT,DABS,DSIGN
!
      IERR = 0
      IF (N .EQ. 1) GO TO 1001
!
      DO 100 I = 2, N
  100 E(I-1) = E(I)
!
      F = 0.0D0
      B = 0.0D0
      E(N) = 0.0D0
!
      DO 240 L = 1, N
	 J = 0
	 H = MACHEP * (DABS(D(L)) + DABS(E(L)))
	 IF (B .LT. H) B = H
!
	 DO 110 M = L, N
	    IF (DABS(E(M)) .LE. B) GO TO 120
!
  110    CONTINUE
!
  120    IF (M .EQ. L) GO TO 220
  130    IF (J .EQ. 30) GO TO 1000
	 J = J + 1
!
	 L1 = L + 1
	 G = D(L)
	 P = (D(L1) - G) / (2.0D0 * E(L))
	 R = DSQRT(P*P+1.0D0)
	 D(L) = E(L) / (P + DSIGN(R,P))
	 H = G - D(L)
!
	 DO 140 I = L1, N
  140    D(I) = D(I) - H
!
	 F = F + H
!
	 P = D(M)
	 C = 1.0D0
	 S = 0.0D0
	 MML = M - L
!
	 DO 200 II = 1, MML
	    I = M - II
	    G = C * E(I)
	    H = C * P
	    IF (DABS(P) .LT. DABS(E(I))) GO TO 150
	    C = E(I) / P
	    R = DSQRT(C*C+1.0D0)
	    E(I+1) = S * P * R
	    S = C / R
	    C = 1.0D0 / R
	    GO TO 160
  150       C = P / E(I)
	    R = DSQRT(C*C+1.0D0)
	    E(I+1) = S * E(I) * R
	    S = 1.0D0 / R
	    C = C * S
  160       P = C * D(I) - S * G
	    D(I+1) = H + S * (C * G + S * D(I))
!
	    DO 180 K = 1, N
	       H = Z(K,I+1)
	       Z(K,I+1) = S * Z(K,I) + C * H
	       Z(K,I) = C * Z(K,I) - S * H
  180       CONTINUE
!
  200    CONTINUE
!
	 E(L) = S * P
	 D(L) = C * P
	 IF (DABS(E(L)) .GT. B) GO TO 130
  220    D(L) = D(L) + F
  240 CONTINUE
!
      DO 300 II = 2, N
	 I = II - 1
	 K = I
	 P = D(I)
!
	 DO 260 J = II, N
	    IF (D(J) .GE. P) GO TO 260
	    K = J
	    P = D(J)
  260    CONTINUE
!
	 IF (K .EQ. I) GO TO 300
	 D(K) = D(I)
	 D(I) = P
!
	 DO 280 J = 1, N
	    P = Z(J,I)
	    Z(J,I) = Z(J,K)
	    Z(J,K) = P
  280    CONTINUE
!
  300 CONTINUE
!
      GO TO 1001
!
 1000 IERR = L
 1001 RETURN
!
      END
!
!
!
! = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
! @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

        SUBROUTINE SPLIN0(X,Y,N,Y2)
        PARAMETER (NMAX=50)
        DIMENSION X(N),Y(N),Y2(N),U(NMAX)

        Y2(1)=0.
        U(1)=0.

        DO      I=2,N-1
                SIG=(X(I)-X(I-1))/(X(I+1)-X(I-1))
                P=SIG*Y2(I-1)+2.
                Y2(I)=(SIG-1.)/P
                U(I)=(6.*((Y(I+1)-Y(I))/(X(I+1)-X(I))-(Y(I)-Y(I-1))/(X(I)-X(I-1)))/(X(I+1)-X(I-1))-SIG*U(I-1))/P
        END DO

        QN=0.
        UN=0.

        Y2(N)=(UN-QN*U(N-1))/(QN*Y2(N-1)+1.)

        DO      K=N-1,1,-1
                Y2(K)=Y2(K)*Y2(K+1)+U(K)
        END DO

        RETURN
        END

! @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

      SUBROUTINE SPLINT(XA,YA,Y2A,N,X,Y)
      DIMENSION XA(N),YA(N),Y2A(N)
      KLO=1
      KHI=N
1     IF (KHI-KLO.GT.1) THEN
        K=(KHI+KLO)/2
        IF(XA(K).GT.X)THEN
          KHI=K
        ELSE
          KLO=K
        ENDIF
      GOTO 1
      ENDIF
      H=XA(KHI)-XA(KLO)
      IF (H.EQ.0.) PAUSE 'Bad XA input.'
      A=(XA(KHI)-X)/H
      B=(X-XA(KLO))/H
      Y=A*YA(KLO)+B*YA(KHI)+((A**3-A)*Y2A(KLO)+(B**3-B)*Y2A(KHI))*(H**2)/6.
      RETURN
      END

! @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@